import * as html from 'html-escaper';
import { Converter } from 'typedoc/dist/lib/converter';
import { Component, ConverterComponent } from 'typedoc/dist/lib/converter/components';
import { PageEvent, MarkdownEvent } from 'typedoc/dist/lib/output/events';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

var MermaidPlugin_1;
/**
 * Mermaid plugin component.
 */
let MermaidPlugin = MermaidPlugin_1 = class MermaidPlugin extends ConverterComponent {
    /**
     * 1. Load mermaid.js library.
     * 2. Initialize mermaid.
     * 3. Close body tag.
     */
    static customScriptsAndBodyClosingTag = `
    <script
      src="https://unpkg.com/mermaid/dist/mermaid.min.js"
    ></script>
    <script>
    mermaid.initialize({
      startOnLoad: true,
    });
    </script>
    </body>
  `;
    static markdownStartMermaid = '\n```mermaid\n';
    static markdownEndMermaid = '\n```\n';
    /**
     * filter logic for Comment exist
     */
    static filterComment(comment) {
        return comment !== undefined && !!comment;
    }
    /**
     * filter logic for CommentTags exist
     */
    static filterCommentTags(tags) {
        return tags !== undefined && !!tags;
    }
    /**
     * return turn when tag's paramName is 'mermaid'
     */
    static isMermaidCommentTag(tag) {
        return tag.tagName === 'mermaid';
    }
    /**
     * get CommentTags for using `@mermaid` annotation from Context.
     */
    static mermaidTags(context) {
        return Object.values(context.project.reflections) // get reflection from context
            .map((reflection) => reflection.comment) // get Comment from Reflection
            .filter(this.filterComment) // filter only comment exist
            .map((comment) => comment.tags) // get CommentTags from Comment
            .filter(this.filterCommentTags) // filter only CommentTags exist
            .reduce((a, b) => a.concat(b), []) // merge all CommentTags
            .filter(this.isMermaidCommentTag); // filter tag that paramName is 'mermaid'
    }
    /**
     * Regex literal that matches body closing tag.
     */
    BODY_CLOSING_TAG = /<\/body>/;
    /**
     * The first line of text wraps h4.
     * The other wraps by div classed mermaid.
     */
    convertCommentTagText(tagText) {
        const texts = tagText.split('\n');
        // take first line
        const title = texts.shift();
        // the other
        const mermaid = texts.join('\n');
        return `#### ${title} \n\n <div class="mermaid">${mermaid}</div>`;
    }
    /**
     * Insert custom script before closing body tag.
     */
    convertPageContents(contents) {
        if (this.BODY_CLOSING_TAG.test(contents)) {
            return contents.replace(this.BODY_CLOSING_TAG, MermaidPlugin_1.customScriptsAndBodyClosingTag);
        }
        return contents;
    }
    /**
     * listen to event on initialization
     */
    initialize() {
        this.listenTo(this.owner, {
            [Converter.EVENT_RESOLVE_BEGIN]: this.onResolveBegin,
        })
            .listenTo(this.application.renderer, {
            [PageEvent.END]: this.onPageEnd,
        })
            .listenTo(this.application.renderer, {
            [MarkdownEvent.PARSE]: this.onParseMarkdown,
        }, undefined, 100);
    }
    /**
     * Triggered when the converter begins converting a project.
     */
    onResolveBegin(context) {
        MermaidPlugin_1.mermaidTags(context).forEach((tag) => {
            // convert
            tag.text = this.convertCommentTagText(tag.text);
        });
    }
    /**
     * Triggered after a document has been rendered, just before it is written to disc.
     * Remove duplicate lines to tidy up output
     */
    onPageEnd(page) {
        if (page.contents !== undefined) {
            // convert
            page.contents = this.convertPageContents(page.contents);
        }
    }
    onParseMarkdown(event) {
        event.parsedText = this.replaceMarkdownMermaidCodeBlocks(event.parsedText);
    }
    replaceMarkdownMermaidCodeBlocks(s) {
        let out = '';
        let i = 0;
        for (let j = s.indexOf(MermaidPlugin_1.markdownStartMermaid, i); j >= 0; j = s.indexOf(MermaidPlugin_1.markdownStartMermaid, i)) {
            const start = j + MermaidPlugin_1.markdownStartMermaid.length;
            const end = s.indexOf(MermaidPlugin_1.markdownEndMermaid, start);
            out += `${s.slice(i, j + 1)}<div class="mermaid">${html.escape(s.slice(start, end))}</div>`;
            i = end + MermaidPlugin_1.markdownEndMermaid.length - 1;
        }
        return out + s.slice(i);
    }
};
MermaidPlugin = MermaidPlugin_1 = __decorate([
    Component({ name: 'mermaid' })
], MermaidPlugin);

function load(PluginHost) {
    const app = PluginHost.owner;
    if (app.converter.hasComponent('mermaid')) {
        return;
    }
    /**
     * Add the plugin to the converter instance
     */
    app.converter.addComponent('mermaid', new MermaidPlugin(app.converter));
}

export { load };
